"""Constants used in the project."""

IMG_FOLDER_NAME = "Images"  # name of the folder with images in each target dataset
MASK_FOLDER_NAME = "Masks"  # name of the folder with masks in each target dataset

TARGET_PATH = "./data/"  # name of the folder, where processed outputs will be placed
